import lightshader.maya.lighting.gui.attributecontrol as acgui
from lightshader.maya.qt.helper import mayaMainWindow, wrapinstance, getQObject
from lightshader.maya.api.api import API
from lightshader.qt.interface import QtCore, QtWidgets
import pymel.core as pmc
from lightshader.maya.core import Scene, Renderer, Attribute
import random


class AttributeControl(acgui.AttributeControl):
    def __init__(self):
        super(AttributeControl, self).__init__(mayaMainWindow())
        self._populateObjectList()


    def _populateObjectList(self, append=False, recursive=False):

        if not append:
            self.objectList.clear()
        activeSelection = pmc.selected()
        if not activeSelection:
            return pmc.displayWarning('Please make a valid selection')

        items = list()

        if not recursive:
            for obj in API.getHierarchyOfSelection():
                if obj not in [item.text() for item in self.objectList.allItems()]:
                    items.append(obj)
        else:
            for obj in activeSelection:
                items.append(obj.fullPath())

        self.objectList.addItems(items)
        self.objectList.selectAll()

    def objectControlEvent(self, button, *args, **kwargs):
        super(AttributeControl, self).objectControlEvent(button, args, kwargs)
        mode = button.property('mode')
        if mode == 'refresh':
            self._populateObjectList()
        elif mode == 'add':
            self._populateObjectList(append=True)
        elif mode == 'clear':
            self.objectList.clear()
        elif mode == 'recursive':
            self._populateObjectList(append=False, recursive=True)

    def controlSignalEvent(self, button, *args, **kwargs):
        super(AttributeControl, self).controlSignalEvent(button, args, kwargs)
        text = button.text()
        attribute = button.property('attribute')
        value = button.property('value')
        widget = button.property('control')

        if text == 'Set':
            for item in self.objectList.selectedItems():
                if value is not None:
                    Attribute.set(item.text(), attribute, value)
                elif widget:
                    Attribute.set(item.text(), attribute, widget.value())
        elif text == 'Increment':
            for item in self.objectList.selectedItems():
                currentValue = Attribute.get(item.text(), attribute)
                if widget and currentValue is not None:
                    Attribute.set(item.text(), attribute, currentValue + widget.value())
        elif text == 'Random':
            randomDialog = acgui.RandomDialog(self)
            randomDialog.exec_()
            if randomDialog.value:
                for item in self.objectList.selectedItems():
                    Attribute.set(item.text(), attribute, randomDialog.randomize())
        else:
            for item in self.objectList.selectedItems():
                Attribute.set(item.text(), attribute, value)



